#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   abaqus2neutral.py
#
#       abaqus形式のメッシュファイルをneutral形式に変換
#
#
#   18/03/03    新規作成
#

import sys
import os
import glob
import convertMesh as cm

ls = "\n"


solidElements = ["C3D4", "C3D10", "C3D8", "C3D8R", "C3D20", "C3D20R", "C3D6", "C3D15"]
surfaceElements = ["STRI35", "S3", "S6", "S4R5", "S4", "S8"]


#  readFileLines
#    file内容を読み込み
def readFileLines(fileName):
    f = open(fileName)
    lines = f.readlines()
    f.close()
    return lines

#  writeFileLines
#    file書き込み
def writeFileLines(fileName, lines):
    f = open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()

#  getMeshHeaderConts
#    メッシュのheader内容を取得
def getMeshHeaderConts(lines):
    meshHeaderConts = []
    headerCont = []
    for line in lines:
        if line[0] == "*":
            if len(headerCont) > 0:
                meshHeaderConts.append(headerCont)
            headerCont = [line, []]
        else:
            headerCont[1].append(line)
    meshHeaderConts.append(headerCont)
    return meshHeaderConts

#  deleteSp
#    spaceを削除
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara == " " or chara == ls:
            pass
        else:
            newLine += chara
    return newLine

#  getValName
#    行内からTYPE=AAA等のTYPEの値を取得
def getValName(line, valName):
    valName = valName.upper()
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.upper().find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#  getHeaderKeyword
#    headerのkeywordを取得
def getHeaderKeyword(line):
    keyword = deleteSp(line).split(",")[0]
    return keyword

#
#  createNodeList
#    nodeList作成
def createNodeList(meshHeaderConts):
    nodeLines = []
    for headerCont in meshHeaderConts:
        header = headerCont[0]
        keyword = getHeaderKeyword(header)
        if keyword == "*NODE":
            nodeLines += headerCont[1]
    nodeConts = []
    for line in nodeLines:
        words = deleteSp(line).split(",")
        nodeNo = int(words[0])
        x = float(words[1])
        y = float(words[2])
        z = float(words[3])
        nodeConts.append([nodeNo, x,y,z])
    nodeConts.sort()
    maxNodeNo = nodeConts[-1][0]
    nodeList = [[] for i in range(maxNodeNo+1)]
    for nodeCont in nodeConts:
        nodeNo = nodeCont[0]
        nodeList[nodeNo] = [nodeCont[1:]]
    return nodeList

#
#  createElementList
#    elementListを作成(solid要素のみ）
def createElementList(meshHeaderConts):
    eGrpNames = []
    elementConts = []
    for headerCont in meshHeaderConts:
        header = headerCont[0]
        keyword = getHeaderKeyword(header)
        if keyword == "*ELEMENT":
            elType = getValName(header, "TYPE")
            if (elType in solidElements) == True:
                eGrpName = getValName(header, "ELSET")
                eGrpNames.append(eGrpName)
                for line in headerCont[1]:
                    words = deleteSp(line).split(",")
                    if words[-1] == "":
                        words = words[:-1]
                    elCont = []
                    for word in words:
                        elCont += [int(word)]
                    elementConts += [elCont]
    elementConts.sort()
    maxElNo = elementConts[-1][0]
    elementList = [[] for i in range(maxElNo+1)]
    grpNo = 1
    for elmCont in elementConts:
        elNo = elmCont[0]
        elementList[elNo] = [[grpNo] + elmCont[1:]]
    return [elementList, eGrpNames]

#
#  createSurfaceList
#    surfaceListの作成
def createSurfaceList(meshHeaderConts):
    surfaceConts = []
    for headerCont in meshHeaderConts:
        header = headerCont[0]
        keyword = getHeaderKeyword(header)
        if keyword == "*ELEMENT":
            elType = getValName(header, "TYPE")
            if (elType in surfaceElements) == True:
                for line in headerCont[1]:
                    words = deleteSp(line).split(",")
                    if words[-1] == "":
                        words = words[:-1]
                    surCont = []
                    for word in words:
                        surCont.append(int(word))
                    surfaceConts.append(surCont)
    surfaceConts.sort()
    maxNo = surfaceConts[-1][0]
    surfaceList = [[] for i in range(maxNo+1)]
    surGrpNo = 0
    for surCont in surfaceConts:
        surNo = surCont[0]
        surfaceList[surNo] = [[surGrpNo] + surCont[1:]]
    return surfaceList

#
#  createNodeGroups
#    nodeGrpsの作成
def createNodeGroups(meshHeaderConts):
    nodeGrps = []
    for headerCont in meshHeaderConts:
        header = headerCont[0]
        keyword = getHeaderKeyword(header)
        if keyword == "*NSET":
            grpName = getValName(header, "NSET")
            nodeData = []
            for line in headerCont[1]:
                words = deleteSp(line).split(",")
                if words[-1] == "":
                    words = words[:-1]
                for word in words:
                    nodeNo = int(word)
                    nodeData.append(nodeNo)
            nodeData.sort()
            nodeGrps += [[grpName, nodeData]]
    return nodeGrps

#
#  setElementGroupNo
#    elementNoをセット
def setElementGroupNo(meshHeaderConts, elementList, surfaceList):
    elmGrpNo = 1
    surGrpNo = 1
    elmGrpNames = []
    surGrpNames = []
    for headerCont in meshHeaderConts:
        header = headerCont[0]
        keyword = getHeaderKeyword(header)
        if keyword == "*ELSET":
            elSetName = getValName(header, "ELSET")
            elData = headerCont[1]
            elNo = int(elData[0].split(",")[0])
            if len(elementList[elNo]) > 0:
                #elementをセット
                elmGrpNames.append(elSetName)
                for line in elData:
                    words = deleteSp(line).split(",")
                    for word in words:
                        elNo = int(word)
                        elementList[elNo][0][0] = elmGrpNo
                elmGrpNo += 1
            elif len(surfaceList[elNo]) > 0:
                #surfaceをセット
                surGrpNames.append(elSetName)
                for line in elData:
                    words = deleteSp(line).split(",")
                    for word in words:
                        elNo = int(word)
                        surfaceList[elNo][0][0] = surGrpNo
                surGrpNo += 1
    lastSurGrpNo = surGrpNo
    for i in range(len(surfaceList)):
        surCont = surfaceList[i]
        if len(surCont) > 0:
            if surfaceList[i][0][0] == 0:
                surfaceList[i][0][0] = lastSurGrpNo
    return [elementList, surfaceList, elmGrpNames, surGrpNames]

#
#  renumberList
#
def renumberList(meshList):
    [nodeList, elementList, surfaceList, nodeGrps] = meshList
    nodeNo = 1
    for i in range(len(nodeList)):
        if len(nodeList[i]) > 0:
            nodeList[i] += [nodeNo]
            nodeNo += 1
    maxNodeNo = nodeNo
    #elementListをnewNoで作り直し
    for i in range(len(elementList)):
        if len(elementList[i]) > 0:
            for n in range(1, len(elementList[i][0])):
                nodeNo = elementList[i][0][n]
                newNo = nodeList[nodeNo][-1]
                elementList[i][0][n] = newNo
    #surfaceListをnewNoで作り直し
    for i in range(len(surfaceList)):
        if len(surfaceList[i]) > 0:
            for n in range(1, len(surfaceList[i][0])):
                nodeNo = surfaceList[i][0][n]
                newNo = nodeList[nodeNo][-1]
                surfaceList[i][0][n] = newNo
    #nodeGrpsをnewNoで作り直し
    for i in range(len(nodeGrps)):
        for j in range(len(nodeGrps[i][1])):
            nodeNo = nodeGrps[i][1][j]
            newNo = nodeList[nodeNo][-1]
            nodeGrps[i][1][j] = newNo
    newNodeList = [[]]
    for i in range(len(nodeList)):
        if len(nodeList[i]) > 0:
            newNodeList.append([nodeList[i][0]])
    newElementList = [[]]
    for i in range(len(elementList)):
        if len(elementList[i]) > 0:
            newElementList.append([elementList[i][0]])
    newSurfaceList = [[]]
    for i in range(len(surfaceList)):
        if len(surfaceList[i]) > 0:
            newSurfaceList.append([surfaceList[i][0]])
    meshList = [newNodeList, newElementList, newSurfaceList, nodeGrps]
    return meshList

#
#  reorderList
#    element, surfaceListのnode順をabaqusからneutralに修正
def reorderList(meshList):
    [nodeList, elementList, surfaceList, nodeGrps] = meshList
    for i in range(len(elementList)):
        if len(elementList[i]) > 0:
            nodes = elementList[i][0][1:]
            [name, newNodes] = cm.aba2neu_el(nodes)
            elementList[i][0][1:] = newNodes
    for i in range(len(surfaceList)):
        if len(surfaceList[i]) > 0:
            nodes = surfaceList[i][0][1:]
            [name, newNodes] = cm.aba2neu_el(nodes)
            surfaceList[i][0][1:] = newNodes
    meshList = [nodeList, elementList, surfaceList, nodeGrps]
    return meshList

#
#  createMeshList
#    node, element, surfaceのlistを作成
def createMeshList(meshHeaderConts):
    nodeList = createNodeList(meshHeaderConts)
    [elementList, eGrpNames] = createElementList(meshHeaderConts)
    surfaceList = createSurfaceList(meshHeaderConts)
    nodeGrps = createNodeGroups(meshHeaderConts)
    [elementList, surfaceList, elmGrpNames, surGrpNames] = setElementGroupNo(meshHeaderConts, elementList, surfaceList)
    if len(elmGrpNames) == 0:
        elmGrpNames = eGrpNames
    meshList = [nodeList, elementList, surfaceList, nodeGrps]
    meshList = renumberList(meshList)
    meshList = reorderList(meshList)
    return [meshList, elmGrpNames, surGrpNames]

#
#  createNeuLines
#    meshListからneutral形式の行を作成
def createNeuLines(meshList):
    [nodeList, elementList, surfaceList, nodeGrps] = meshList
    lines = []
    nNode = len(nodeList) - 1
    lines += [str(nNode) + ls]
    for i in range(1, len(nodeList)):
        nodeCont = nodeList[i][0]
        words = []
        for val in nodeCont:
            word = str(val)
            words.append(word)
        line = " ".join(words) + ls
        lines += [line]
    nElement = len(elementList) - 1
    lines += [str(nElement) + ls]
    for i in range(1, len(elementList)):
        elCont = elementList[i][0]
        words = []
        for val in elCont:
            words.append(str(val))
        line = " ".join(words) + ls
        lines += [line]
    nSurface = len(surfaceList) - 1
    lines += [str(nSurface) + ls]
    for i in range(1, len(surfaceList)):
        surCont = surfaceList[i][0]
        words = []
        for val in surCont:
            words.append(str(val))
        line = " ".join(words) + ls
        lines += [line]
    return lines

#
#  abaqus2neutral
#    abaqusからneutral形式に変換
def abaqus2neutral(abaFileName, neuFileName):
    lines = readFileLines(abaFileName)
    meshHeaderConts = getMeshHeaderConts(lines)
    [meshList, surGrpNames] = createMeshList(meshHeaderConts)
    lines = createNeuLines(meshList, elGrpNames, surGrpNames)
    writeFileLines(neuFileName, lines)
    


if __name__=='__main__':
    abaFileName = sys.argv[1]
    words = abaFileName.split(".")
    if words[-1] != "inp":
        abaFileName += ".inp"
    neuFileName = ".".join(abaFileName.split(".")[:-1]) + ".mesh"
    abaqus2neutral(abaFileName, neuFileName)


