#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   abaqus2vol.py
#
#       abaqus形式のメッシュファイルをvol形式に変換
#
#
#   18/03/03    新規作成
#

import sys
import os
import glob
import convertMesh as cm
import abaqus2neutral as an

ls = "\n"


solidElements = ["C3D4", "C3D10", "C3D8", "C3D8R", "C3D20", "C3D20R", "C3D6", "C3D15"]
surfaceElements = ["STRI35", "S3", "S6", "S4R5", "S4", "S8"]


#  readFileLines
#    file内容を読み込み
def readFileLines(fileName):
    f = open(fileName)
    lines = f.readlines()
    f.close()
    return lines

#  writeFileLines
#    file書き込み
def writeFileLines(fileName, lines):
    f = open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()

#  deleteSameData
#    同じdataを削除する
def deleteSameData(dataList):
    if len(dataList) == 0:
        return []
    dataList.sort()
    newData = [dataList[0]]
    for data in dataList:
        if data != newData[-1]:
            newData.append(data)
    return newData

#
#  createVolLines
#    meshListからvol形式の行を作成
def createVolLines(meshList, elmGrpNames, surGrpNames):
    [nodeList, elementList, surfaceList, nodeGrps] = meshList
    lines = []
    #header作成
    lines += ["mesh3d2" + ls]
    lines += ["dimension" + ls]
    lines += ["3" + ls]
    lines += ["geomtype" + ls]
    lines += ["0" + ls]
    lines += [ls]
    lines += ["# surfnr    bcnr   domin  domout      np      p1      p2      p3" + ls]
    #surface
    surGrpNos = []
    lines += ["surfaceelements" + ls]
    nSurface = len(surfaceList) - 1
    lines += [str(nSurface) + ls]
    for i in range(1, len(surfaceList)):
        n = surfaceList[i][0][0] + 1
        surGrpNos.append(n)
        grpNo = str(n)
        bcnr = str(int(grpNo) - 1)
        domin = bcnr
        domout = "0"
        np = str(len(surfaceList[i][0][1:]))
        nodes = surfaceList[i][0][1:]
        words = []
        for nodeNo in nodes:
            words.append(str(nodeNo))
        line = " " + " ".join([grpNo, bcnr, domin, domout, np] + words) + ls
        lines += [line]
    lines += [ls]
    lines += [ls]
    surGrpNos = deleteSameData(surGrpNos)
    #element
    lines += ["#  matnr      np      p1      p2      p3      p4" + ls]
    lines += ["volumeelements" + ls]
    nElement = len(elementList) - 1
    lines += [str(nElement) + ls]
    for i in range(1, len(elementList)):
        matnr = str(elementList[i][0][0])
        np = str(len(elementList[i][0][1:]))
        nodes = elementList[i][0][1:]
        words = []
        for nodeNo in nodes:
            words.append(str(nodeNo))
        line = " ".join([matnr, np] + words) + ls
        lines += [line]
    lines += [ls]
    lines += [ls]
    #edge
    lines += ["# surfid  0   p1   p2   trignum1    trignum2   domin/surfnr1    domout/surfnr2   ednr1   dist1   ednr2   dist2" + ls]
    lines += ["edgesegmentsgi2" + ls]
    lines += ["0" + ls]
    lines += [ls]
    lines += [ls]
    #node
    lines += ["#          X             Y             Z" + ls]
    lines += ["points" + ls]
    nNode = len(nodeList) - 1
    lines += [str(nNode) + ls]
    for i in range(1, len(nodeList)):
        x = nodeList[i][0][0]
        y = nodeList[i][0][1]
        z = nodeList[i][0][2]
        line = str(x) + " " + str(y) + " " + str(z) + ls
        lines += [line]
    lines += [ls]
    lines += [ls]
    #faceColor
    lines += ["#   Surfnr     Red     Green     Blue" + ls]
    lines += ["face_colours" + ls]
    lines += [str(len(surGrpNos)) + ls]
    for grpNo in surGrpNos:
        lines += ["       " + str(grpNo) + "   0.00000000   1.00000000   0.00000000" + ls]
    lines += [ls]
    lines += [ls]
    #end
    lines += ["endmesh" + ls]
    lines += [ls]
    lines += [ls]
    #elmGrps
    elmGrpNo = 1
    for elmGrpName in elmGrpNames:
        lines += ["# *elementGroup " + str(elmGrpNo) + " " + elmGrpName + ls]
        elmGrpNo += 1
    #surGrps
    surGrpNo = 2
    for surGrpName in surGrpNames:
        lines += ["# *surfaceGroup " + str(surGrpNo) + " " + surGrpName + ls]
        surGrpNo += 1
    #nodeGrps
    maxNo = 10
    for nodeGrp in nodeGrps:
        lines += ["# *nodeGroup " + nodeGrp[0] + ls]
        nodeData = nodeGrp[1]
        i = 1
        line = "# "
        for nodeNo in nodeData:
            line += " " + str(nodeNo)
            if i >= maxNo:
                lines += [line + ls]
                i = 0
                line = "# "
            i += 1
        if line != "# ":
            lines += [line + ls]
    return lines

#
#  abaqus2vol
#    abaqusからvol(netgen)形式に変換
def abaqus2vol(abaFileName, volFileName):
    lines = readFileLines(abaFileName)
    meshHeaderConts = an.getMeshHeaderConts(lines)
    [meshList, elmGrpNames, surGrpNames] = an.createMeshList(meshHeaderConts)
    lines = createVolLines(meshList, elmGrpNames, surGrpNames)
    writeFileLines(volFileName, lines)
    


if __name__=='__main__':
    abaFileName = sys.argv[1]
    words = abaFileName.split(".")
    if words[-1] != "inp":
        abaFileName += ".inp"
    volFileName = ".".join(abaFileName.split(".")[:-1]) + ".vol"
    abaqus2vol(abaFileName, volFileName)


