#!/usr/bin/env python  
# -*- coding: utf-8 -*-    
#
#   getGroupFromVolStl.py
#
#       vol形式のpreMesh、partMeshメッシュから各Groupを作成し、neutaral形式で保存する。
#       nodeGroupの形状は、stlから読み取る。
#       表面形状は、表面stl形状にfitさせるので、精細な形状が再現できる。
#
#   18/02/01    新規作成
#
#

import sys
import os
import glob
import geometryFunc as geo
import convertMesh as cm
import setGroupsFromPreMesh as setGrp

ls = "\n"
numlevels = 0

#  readFileLines
#    fileをlinesで読み込む
def readFileLines(fileName):
    f = open(fileName)
    lines = f.readlines()
    f.close()
    return lines

#  writeFileLines
#    fileを書き込む
def writeFileLines(fileName, lines):
    f = open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()

#  findLine
#    lineを検索する
def findLine(keyword, lines):
    for i in range(len(lines)):
        line = lines[i]
        if line[:len(keyword)] == keyword:
            break
    return i

#  deleteSameData
#    同じdataを削除する
def deleteSameData(dataList):
    if len(dataList) == 0:
        return []
    dataList.sort()
    newData = [dataList[0]]
    for data in dataList:
        if data != newData[-1]:
            newData.append(data)
    return newData

#  getNeutralContents
#    neutral形式の内容を取得
def getNeutralContents(lines, i):
    num = int(lines[i])
    st = i + 1
    ed = i + num + 1
    newLines = lines[st:ed]
    return newLines

#  getGroupLines
#    *nodeGroup,*surfaceGroup行を取得
def getGroupLines(lines, i):
    newLines = []
    if i+1 == len(lines):
        return newLines
    flag = 0
    for line in lines[i:]:
        if line[:len("# *")] == "# *":
            newLines += [line[2:]]
            flag = 1
        else:
            if flag == 1 and line[:len("# ")] == "# ":
                newLines += [line[2:]]
    return newLines

#
#  getVolLines
#    volMesh内の行を分割して取得
def getVolLines(lines):
    i = 0
    grpLines = []
    while i < len(lines):
        line = lines[i]
        if line[:len("surfaceelements")] == "surfaceelements":
            surLines = getNeutralContents(lines, i+1)
            i += int(lines[i+1].split()[0])
        elif line[:len("volumeelements")] == "volumeelements":
            elLines = getNeutralContents(lines, i+1)
            i += int(lines[i+1].split()[0])
        elif line[:len("points")] == "points":
            nodeLines = getNeutralContents(lines, i+1)
            i += int(lines[i+1].split()[0])
        elif line[:len("# *")] == "# *":
            grpLines = getGroupLines(lines, i-1)
            i = len(lines)
        i += 1
    return [surLines, elLines, nodeLines, grpLines]

#
#  readVolMesh
#    vol形式のメッシュを読み込む
def readVolMesh(volFile):
    lines = readFileLines(volFile)
    #neutralの内容を取得
    [surLines, elLines, nodeLines, grpLines] = getVolLines(lines)

    #nodeList作成 nodeList[i] -> [[x,y,z], [eli, elj...], [sui, suj...]]
    #                             座標      要素No         surfaceNo
    #  要素No:    surfaceが属する要素の面番号を取得するため
    #  surfaceNo: 表面の節点を取得する為
    nodeList = [ [] for i1 in range(len(nodeLines) + 1) ]
    for i in range(len(nodeLines)):
        line = nodeLines[i]
        words = line.split()
        loc = [float(words[0]), float(words[1]), float(words[2])]
        nodeList[i+1] = [loc, [], []]
    #elementList作成 elementList -> [[grp, node1, node2,...]]
    elementList = [ [] for i1 in range(len(elLines) + 1) ]
    for i in range(len(elLines)):
        line = elLines[i]
        words = line.split()
        grp = words[0]
        nEl = words[1]
        element = [ [] for i1 in range(int(nEl)+1) ]
        element[0] = int(grp)
        n = 1
        for word in words[2:]:
            nodeNo = int(word)
            element[n] = nodeNo
            nodeList[nodeNo][1] += [i+1]     #nodeListに要素Noを保存
            n += 1
        elementList[i+1] = [element]
    #surfaceList作成  surfaceList -> [[grp, node1, node2,...], [eli, elj...]]
    #                                   表面要素内容             要素No
    #  要素No: surfaceが属する面番号を取得するため
    surfaceList = [ [] for i1 in range(len(surLines) + 1) ]
    for i in range(len(surLines)):
        line = surLines[i]
        words = line.split()
        grp = words[0]
        nP = words[4]
        surface = [ [] for i1 in range(int(nP) + 1) ]
        surface[0] = int(grp)
        n = 1
        for word in words[5:8]:
            nodeNo = int(word)
            surface[n] = nodeNo
            nodeList[nodeNo][2] += [i+1]   #nodeListにsurfaceNoを保存
            n += 1
        surfaceList[i+1] = [surface, []]
    #surfaceListに要素Noを追加
    for i in range(len(surfaceList)):
        surface = surfaceList[i]
        if len(surface) > 0:
            #surfaceのnodeNoを取得
            nodes = surface[0][1:]
            if len(nodes) == 3 or len(nodes) == 6:
                #三角形
                nodes = nodes[:3]
            else:
                #四角形
                nodes = nodes[:4]
            #nodeから属する要素Noを取得
            elms = []
            for node in nodes:
                elms += nodeList[node][1]
            #同じ要素がnode数分ある要素を選択
            elms.sort()
            before = elms[0]
            n = 1
            for elm in elms[1:]:
                if elm == before:
                    n += 1
                else:
                    #要素数がnode数分ある場合は、break
                    if n >= len(nodes):
                        break
                    n = 1
                    before = elm
            if n >= len(nodes):
                #要素を取得
                surfaceList[i][1] += [before]
    #groupの読み込み
    grps = []
    nodeData = []
    flag = ""
    for i in range(len(grpLines)):
        line = grpLines[i]
        words = line.split()
        if words[0] == "*nodeGroup":
            if len(nodeData) > 0:
                grps += [[header, nodeData]]
            header = line
            nodeData = []
            flag = "node"
        elif words[0] == "*surfaceGroup":
            grps += [[line, []]]
            flag = "surface"
        elif words[0] == "*elementGroup":
            grps += [[line, []]]
        else:
            if flag == "node":
                for word in words:
                    nodeData.append(int(word))
    if len(nodeData) > 0:
        grps += [[header, nodeData]]
    return [nodeList, elementList, surfaceList, grps]

#
#  createElementGroupsFromList
#    elementListからelGgroupを取得
def createElementGroupsFromList(elementList, preGrps):
    #最大のelGrpNoを取得
    maxNo = -1
    for i in range(len(elementList)):
        if len(elementList[i]) > 0:
            elGrp = elementList[i][0][0]
            if maxNo < elGrp:
                maxNo = elGrp
    #listを準備
    elGroups = []
    for i in range(maxNo+1):
        elGroups.append([])
    #要素の内容を取得
    n = 0
    for element in elementList:
        if len(element) > 0:
            elCont = element[0]
            elGrp = elCont[0]
            elNodes = elCont[1:]
            elData = [n] + elNodes
            elGroups[elGrp].append(elData)
        n += 1
    #要素名を取得
    elNames = [ [] for i in range(maxNo+1) ]
    for preGrp in preGrps:
        header = preGrp[0]
        words = header.split()
        if words[0] == "*elementGroup":
            grpNo = int(words[1])
            grpName = words[2]
            elNames[grpNo] = grpName
    #要素名を追加
    elGrps = []
    n = 0
    for elGroup in elGroups:
        if len(elGroup) > 0:
            if len(elNames[n]) == 0:
                name = "elm_" + str(n)
            else:
                name = elNames[n]
            elGrps.append([name, elGroup])
        n += 1
    return elGrps

#
#  createSurfaceNodeGroupsFromList
#    surfaceListからsurfaceGroupとnodeGroupを取得
def createSurfaceNodeGroupsFromList(surfaceList):
    #最大のnodeGrpNoを取得
    maxNo = -1
    for i in range(len(surfaceList)):
        if len(surfaceList[i]) > 0:
            ndGrp = surfaceList[i][0][0]
            if maxNo < ndGrp:
                maxNo = ndGrp
    #listを準備
    suGrps = []
    ndGrps = []
    for i in range(maxNo+1):
        ndGrps.append([])
        suGrps.append([])
    #nodeGrpのnodeNoを取得
    for n in range(len(surfaceList)):
        surface = surfaceList[n]
        if len(surface) > 0:
            grpNo = surface[0][0]
            nodes = surface[0][1:]
            ndGrps[grpNo] += nodes
            suGrps[grpNo] += [n]
    #nodeNoを整形（同じnodeNoを削除）
    for n in range(len(ndGrps)):
        grpNodes = ndGrps[n]
        if len(grpNodes) > 1:
            grpNodes = deleteSameData(grpNodes)
            ndGrps[n] = grpNodes
    #grp名を挿入（最後のgroupは、分割面の為捨てる）
    #  surfaceGroupsを取得
    surfaceGroups = []
    for n in range(len(suGrps[:-1])):   #最後のgroupを捨てる
        if len(suGrps[n]) > 0:
            name = "surface_" + str(n)
            surfaceGroups.append([name, suGrps[n]])
    #  nodeGroupsを取得
    nodeGroups = []
    for n in range(len(ndGrps[:-1])):   #最後のgroupを捨てる
        if len(ndGrps[n]) > 0:
            name = "node_" + str(n)
            nodeGroups.append([name, ndGrps[n]])
    return [surfaceGroups, nodeGroups]

#
#  createSurfaceNodeGroupsFromPremesh
#    preGrpsからsurfaceGroupとnodeGroupを取得
def createSurfaceNodeGroupsFromPremesh(preMeshList, preGrps, meshList, minMaxLoc):

    def getNodesInFaces(faces, preMeshList, minMaxLoc, meshList, grpName):
        [preNodeList, preElemList, preSurList] = preMeshList
        [nodeList, elementList, surfaceList] = meshList
        #三角形の座標を取得
        preTris = []
        for faceNo in faces:
            nodes = preSurList[faceNo][0][1:4]
            loc0 = preNodeList[nodes[0]][0]
            loc1 = preNodeList[nodes[1]][0]
            loc2 = preNodeList[nodes[2]][0]
            tri = [loc0, loc1, loc2]
            preTris += [tri]
        #表面要素のaddressを作成
        surAdd = setGrp.setSurfaceAddress(nodeList, surfaceList, minMaxLoc)
        #設定or未設定flag
        surFlag = [0] * len(surfaceList)
        #procMeshのsurfaceNoを取得
        selSurNo = []
        for preTri in preTris:
            #blockを取得
            #blockを取得
            xx = [preTri[0][0], preTri[1][0], preTri[2][0]]
            yy = [preTri[0][1], preTri[1][1], preTri[2][1]]
            zz = [preTri[0][2], preTri[1][2], preTri[2][2]]
            minx = min(xx); maxx = max(xx)
            miny = min(yy); maxy = max(yy)
            minz = min(zz); maxz = max(zz)
            [ax, ay, az] = setGrp.getLocAddress([minx, miny, minz], minMaxLoc)
            [bx, by, bz] = setGrp.getLocAddress([maxx, maxy, maxz], minMaxLoc)
            #ax, bxを調整する
            [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
            [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
            #[ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
            #[bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
            #block内addressを取得
            blockSur = setGrp.getLocAddInBlock(surAdd, [ax, ay, az], [bx, by, bz])
            #block内のloop
            for surNo in blockSur:
                #設定or未設定
                if surFlag[surNo] == 0:
                    #faceの角度をチェック
                    nodes = surfaceList[surNo][0][1:4]
                    loc0 = nodeList[nodes[0]][0]
                    loc1 = nodeList[nodes[1]][0]
                    loc2 = nodeList[nodes[2]][0]
                    triLoc = [loc0, loc1, loc2]
                    cosAng = geo.facesAngle(preTri, triLoc)
                    if cosAng > 0.9659:
                        #face重心と距離チェック
                        centerLoc = geo.triG(triLoc)
                        l = geo.isPointInFace(preTri, centerLoc)
                        tol = geo.length(preTri[0], preTri[1]) * 0.5
                        if 0.0 <= l and l < tol:
                            selSurNo.append(surNo)
                            surFlag[surNo] = 1

                        """
                        [flg, l] = geo.isPointInFaceLarge(preTri, centerLoc)
                        if flg >= 0:
                            tol = geo.length(preTri[0], preTri[1]) * 0.5
                            if 0.0<= l and l < tol:
                                selSurNo.append(surNo)
                                if flg == 0:
                                    surFlag[surNo] = 1
                        """
        newNodes = []
        for surNo in selSurNo:
            nodes = surfaceList[surNo][0][1:]
            newNodes += nodes
        newNodes = deleteSameData(newNodes)
        return newNodes

    def getNodesInEdges(edges):
        pass

    def getNodesInPoints(points):
        pass

    def getNewNodeData(preMeshList, nodeData, meshList, minMaxLoc, grpName):
        [preNodeList, preElemList, preSurList] = preMeshList
        [nodeList, elementList, surfaceList] = meshList
        #nodeに属する全表面要素を取得
        preSurs = []
        for preNode in nodeData:
            preSurs += preNodeList[preNode][2]
        #全表面要素を仕分けする
        preSurs.sort()
        points = []
        edges = []
        faces = []
        #仕分け
        before = preSurs[0]
        i = 1
        for sur in preSurs[1:]:
            if sur == before:
                i += 1
            else:
                if i == 1:
                    points.append(before)
                elif i == 2:
                    edges.append(before)
                else:
                    faces.append(before)
                i = 1
                before = sur
        #抜け出た後の最後の仕分け
        if i == 1:
            points.append(before)
        elif i == 2:
            edges.append(before)
        else:
            faces.append(before)

        #取得後の処理（nodeNoを取得）
        if len(faces) == 0 and len(edges) == 0:
            #pointsでnodesを取得
            nodes = getNodesInPoints(points)
        elif len(faces) == 0:
            #egesでnodesを取得
            nodes = getNodesInEdges(edges)
        else:
            #facesでnodesを取得
            nodes = getNodesInFaces(faces, preMeshList, minMaxLoc, meshList, grpName)
        return nodes

    [nodeList, elementList, surfaceList] = meshList
    #nodeGroupを取得
    nodeGroups = []
    for preGrp in preGrps:
        words = preGrp[0].split()
        if words[0] == "*nodeGroup":
            grpName = words[1]
            nodeData = preGrp[1]
            newNodeData = getNewNodeData(preMeshList, nodeData, meshList, minMaxLoc, grpName)
            nodeGroups += [[grpName, newNodeData]]
    #surfaceGroupsを取得
    surfaceGroups = []
    #  maxNoを取得
    maxNo = -1
    for i in range(1, len(surfaceList)):
        if maxNo < surfaceList[i][0][0]:
            maxNo = surfaceList[i][0][0]
    #  surNo毎に取得
    surGrpNos = [ [] for i in range(maxNo+1)]
    for i in range(1, len(surfaceList)):
        grpNo = surfaceList[i][0][0]
        surGrpNos[grpNo] += [i]
    #  surNoとsurNameを取得
    surGrpNoNames = []
    for preGrp in preGrps:
        words = preGrp[0].split()
        if words[0] == "*surfaceGroup":
            grpNo = int(words[1])
            grpName = words[2]
            surGrpNoNames.append([grpNo, grpName])
    #  surfaceGroupを取得
    for [grpNo, grpName] in surGrpNoNames:
        grpData = surGrpNos[grpNo]
        surfaceGroups += [[grpName, grpData]]
    return [surfaceGroups, nodeGroups]

#
#  getStlTriLoc
#    stlファイルの三角形座標値を取得
def getStlTriLoc(stlFiles):
    stlLocs = []
    for stlFile in stlFiles:
        triLocs = []
        lines = readFileLines(stlFile)
        i = 0
        while i < len(lines):
            line = lines[i]
            if line.find("vertex") >= 0:
                words = line.split()
                loc0 = [float(words[1]), float(words[2]), float(words[3])]
                words = lines[i+1].split()
                loc1 = [float(words[1]), float(words[2]), float(words[3])]
                words = lines[i+2].split()
                loc2 = [float(words[1]), float(words[2]), float(words[3])]
                triLocs.append([loc0, loc1, loc2])
                i += 2
            i += 1
        name = ".".join(stlFile.split(os.sep)[-1].split(".")[:-1])
        stlLocs.append([name, triLocs])
    return stlLocs    

#
#  getNodeGroupsFromStl
#    stlの三角座標値から表面のnodeGroupを取得
def getNodeGroupsFromStl(nodeList, surfaceList, stlLocs, minMaxLoc, surAdd):
    #設定or未設定のflag
    surFlag = [0 for i in range(len(surfaceList)+1)]
    #stlFileのloop
    nodeGroups = []
    for stlData in stlLocs:
        name = stlData[0]
        stlTris = stlData[1]
        nodeGroup = [name]
        surfaces = []
        #stlDataのloop
        for stlTri in stlTris:
            #blockを取得
            xx = [stlTri[0][0], stlTri[1][0], stlTri[2][0]]
            yy = [stlTri[0][1], stlTri[1][1], stlTri[2][1]]
            zz = [stlTri[0][2], stlTri[1][2], stlTri[2][2]]
            minx = min(xx); maxx = max(xx)
            miny = min(yy); maxy = max(yy)
            minz = min(zz); maxz = max(zz)
            [ax, ay, az] = setGrp.getLocAddress([minx, miny, minz], minMaxLoc)
            [bx, by, bz] = setGrp.getLocAddress([maxx, maxy, maxz], minMaxLoc)
            #ax, bxを調整
            [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
            [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
            [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
            [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
            #block内addressを取得
            blockSur = setGrp.getLocAddInBlock(surAdd, [ax, ay, az], [bx, by, bz])
            #block内のloop
            for surNo in blockSur:
                #設定or未設定
                if surFlag[surNo] == 0:
                    #faceの角度をチェック
                    nodes = surfaceList[surNo][0][1:4]
                    loc0 = nodeList[nodes[0]][0]
                    loc1 = nodeList[nodes[1]][0]
                    loc2 = nodeList[nodes[2]][0]
                    triLoc = [loc0, loc1, loc2]
                    cosAng = geo.facesAngle(stlTri, triLoc)
                    if cosAng > 0.9659:
                        #face重心との垂直距離チェック
                        centerLoc = geo.triG(triLoc)
                        l = geo.isPointInFace(stlTri, centerLoc)
                        ll = geo.length(loc0, loc1)
                        tol = ll * (2**numlevels) * 0.5
                        if 0.0 <= l and l <= tol:
                            #surfaceNoとflagを設定
                            surfaces.append(surNo)
                            surFlag[surNo] = 1
        #surfaceNoからnodeNoを取得
        nodeNos = [0] * len(nodeList)
        for suNo in surfaces:
            nodes = surfaceList[suNo][0][1:]
            for n in nodes:
                nodeNos[n] = 1
        nodeGrp = []
        for n in range(len(nodeNos)):
            if nodeNos[n] == 1:
                nodeGrp.append(n)
        nodeGroup += [nodeGrp]
        nodeGroups.append(nodeGroup)
    return nodeGroups


#
#  getSurGroupsFromStl
#    stlの三角座標値から表面のsurfaceGroupを取得
def getSurGroupsFromStl(nodeList, surfaceList, stlLocs, minMaxLoc, surAdd):
    #設定or未設定のflag
    surFlag = [0 for i in range(len(surfaceList)+1)]
    #stlFileのloop
    surGroups = []
    for stlData in stlLocs:
        name = stlData[0]
        stlTris = stlData[1]
        surGroup = [name]
        surfaces = []
        #stlDataのloop
        for stlTri in stlTris:
            #blockを取得
            xx = [stlTri[0][0], stlTri[1][0], stlTri[2][0]]
            yy = [stlTri[0][1], stlTri[1][1], stlTri[2][1]]
            zz = [stlTri[0][2], stlTri[1][2], stlTri[2][2]]
            minx = min(xx); maxx = max(xx)
            miny = min(yy); maxy = max(yy)
            minz = min(zz); maxz = max(zz)
            [ax, ay, az] = setGrp.getLocAddress([minx, miny, minz], minMaxLoc)
            [bx, by, bz] = setGrp.getLocAddress([maxx, maxy, maxz], minMaxLoc)
            #ax, bxを調整（範囲を±2させて広げる）
            [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
            [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
            [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
            [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
            #block内addressを取得
            blockSur = setGrp.getLocAddInBlock(surAdd, [ax, ay, az], [bx, by, bz])
            #block内のloop
            for surNo in blockSur:
                #設定or未設定
                if surFlag[surNo] == 0:
                    #faceの角度をチェック
                    nodes = surfaceList[surNo][0][1:4]
                    loc0 = nodeList[nodes[0]][0]
                    loc1 = nodeList[nodes[1]][0]
                    loc2 = nodeList[nodes[2]][0]
                    triLoc = [loc0, loc1, loc2]
                    cosAng = geo.facesAngle(stlTri, triLoc)
                    if cosAng > 0.9659:
                        #face重心との垂直距離チェック
                        centerLoc = geo.triG(triLoc)
                        l = geo.isPointInFace(stlTri, centerLoc)
                        ll = geo.length(loc0, loc1)
                        tol = ll * (2**numlevels) * 0.5
                        if 0.0 <= l and l <= tol:
                            #surfaceNoとflagを設定
                            surfaces.append(surNo)
                            surFlag[surNo] = 1
        surGroup += [surfaces]
        surGroups.append(surGroup)
    return surGroups

#
#  remakeSurfaceList
#    surfaceListを再作成する
def remakeSurfaceList(surfaceList, surGroups):
    #maxNoを取得
    maxNo = 0
    for surface in surfaceList:
        if len(surface) > 0:
            if maxNo < surface[0][0]:
                maxNo = surface[0][0]
    #最大surNoを修正（この部分がメッシュの分割面）
    n = maxNo + len(surGroups)
    for i in range(len(surfaceList)):
        if len(surfaceList[i]) > 0:
            if surfaceList[i][0][0] == maxNo:
                surfaceList[i][0][0] = n
    #stlのsurGroupsのNoを設定
    n = maxNo
    for i in range(len(surGroups)):
        surData = surGroups[i]
        if len(surData) > 0:
            surNos = surData[1]
            for surNo in surNos:
                surfaceList[surNo][0][0] = n
        n += 1
    return surfaceList

#
#  moveNodeLocToFitFile
#    表面の節点を移動し、stl形状にfitさせる
def moveNodeLocToFitFile(nodeList, surfaceList, stlLocs, minMaxLoc, surAdd):

    def getMinMove(moveDls):
        if len(moveDls) == 1:
            return moveDls[0]
        magVecs = []
        for moveDl in moveDls:
            magVecs.append([geo.mag(moveDl), moveDl])
        magVecs.sort()
        minDl = magVecs[0][1]
        return minDl

    global numlevels
    #設定or未設定のflag
    surFlag = [0 for i in range(len(surfaceList)+1)]
    #node移動距離保存用のリスト
    #move = [[]] * (len(nodeList)+1)
    move =  [[] for i1 in range(len(nodeList)+1) ]
    #stlFileのloop
    for stlData in stlLocs:
        stlTris = stlData[1]
        #stlDataのloop
        for stlTri in stlTris:
            #blockを取得
            xx = [stlTri[0][0], stlTri[1][0], stlTri[2][0]]
            yy = [stlTri[0][1], stlTri[1][1], stlTri[2][1]]
            zz = [stlTri[0][2], stlTri[1][2], stlTri[2][2]]
            minx = min(xx); maxx = max(xx)
            miny = min(yy); maxy = max(yy)
            minz = min(zz); maxz = max(zz)
            [ax, ay, az] = setGrp.getLocAddress([minx, miny, minz], minMaxLoc)
            [bx, by, bz] = setGrp.getLocAddress([maxx, maxy, maxz], minMaxLoc)
            #ax, bxを調整
            [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])   # -1
            [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])   # +1
            #[ax, ay, az] = setGrp.fitIndexA([ax, ay, az])   # -2
            #[bx, by, bz] = setGrp.fitIndexB([bx, by, bz])   # +2
            #block内addressを取得
            blockSur = setGrp.getLocAddInBlock(surAdd, [ax, ay, az], [bx, by, bz])
            #block内のloop
            for surNo in blockSur:
                #設定or未設定
                if surFlag[surNo] == 0:
                    #faceの角度をチェック
                    nodes = surfaceList[surNo][0][1:4]
                    loc0 = nodeList[nodes[0]][0]
                    loc1 = nodeList[nodes[1]][0]
                    loc2 = nodeList[nodes[2]][0]
                    triLoc = [loc0, loc1, loc2]
                    cosAng = geo.facesAngle(stlTri, triLoc)
                    #if cosAng > 0.9659:    #15°
                    if cosAng > 0.9397:     #20°
                    #if cosAng > 0.8660:     #30°
                        #face重心との垂直距離チェック
                        centerLoc = geo.triG(triLoc)
                        l = geo.isPointInFace(stlTri, centerLoc)
                        ll = geo.length(loc0, loc1)
                        tol = ll * (2 ** numlevels) * 0.5
                        if 0.0 <= l and l < tol:
                            #nodeの移動距離を算出
                            nodes = surfaceList[surNo][0][1:]
                            for node in nodes:
                                loc = nodeList[node][0]
                                dl = geo.vecPointToFace(stlTri, loc)
                                move[node] += [dl]
                            surFlag[surNo] = 1
    #座標を移動
    for i in range(len(move)):
        if len(move[i]) > 0:
            minDl = getMinMove(move[i])
            nodeList[i][0][0] += minDl[0]
            nodeList[i][0][1] += minDl[1]
            nodeList[i][0][2] += minDl[2]
    return nodeList

#
#  getEdgeLinesFromStl
#    stlListからedgeListを取得する
def getEdgeLinesFromStl(nList, sList, lineList):

    def getTriNodeLoc(surNo):
        nodes = sList[surNo][0]
        loc0 = nList[nodes[0]][0]
        loc1 = nList[nodes[1]][0]
        loc2 = nList[nodes[2]][0]
        tri = [loc0, loc1, loc2]
        return tri

    edgeLineNos = []
    for i in range(1, len(lineList)):
        lineNo = i
        surfaces = lineList[lineNo][1]
        flag = 0
        for j in range(len(surfaces)-1):
            for k in range(j+1, len(surfaces)):
                surNo0 = surfaces[j]
                face0 = getTriNodeLoc(surNo0)
                surNo1 = surfaces[k]
                face1 = getTriNodeLoc(surNo1)
                cosAngle = geo.facesAngle(face0, face1)
                if cosAngle < 0.8660:           #30°以上
                    flag = 1
                    break
            if flag == 1:
                break
        if flag == 1:
            edgeLineNos += [lineNo]
    return edgeLineNos

#
#  moveNodeToEdgeOfFitFile
#    fitFile(stl)のedgeに節点を移動する。
def moveNodeToEdgeOfFitFile(nodeList, surfaceList, stlLocs, minMaxLoc):

    def getMinMove(moveDls):
        if len(moveDls) == 1:
            return moveDls[0]
        magVecs = []
        for moveDl in moveDls:
            magVecs.append([geo.mag(moveDl), moveDl])
        magVecs.sort()
        minDl = magVecs[0][1]
        return minDl

    #  isNodeOnEdge
    #    edge上のnodeかどうかcheck
    #    nodeを共有している2面の角度を求め、edge上のnodeかどうか判断
    def isNodeOnEdge(nodeNo, nodeList, surfaceList, sepSurNo):
        ans = False
        surs = nodeList[nodeNo][2]
        #sursから分割面を除く
        surNos = []
        for surNo in surs:
            if surfaceList[surNo][0][0] != sepSurNo:
                surNos += [surNo]
        if len(surNos) < 2:
            return ans
        #面の角度をチェック
        sur0 = surNos[0]
        nodes = surfaceList[sur0][0][1:4]
        loc0 = nodeList[nodes[0]][0]
        loc1 = nodeList[nodes[1]][0]
        loc2 = nodeList[nodes[2]][0]
        face0 = [loc0, loc1, loc2]          #基準のface
        for i in range(1, len(surNos)):
            sur1 = surNos[i]
            nodes = surfaceList[sur1][0][1:4]
            loc10 = nodeList[nodes[0]][0]
            loc11 = nodeList[nodes[1]][0]
            loc12 = nodeList[nodes[2]][0]
            face1 = [loc10, loc11, loc12]
            cosAng = geo.facesAngle(face0, face1)
            if cosAng < 0.8660:             #30°以上？           
                ans = True                  #edge上の節点
                break
        return ans

    global numlevels    
    #stlのedgeを取得
    #全stlの三角形座標を取得
    tris = []
    for stlLoc in stlLocs:
        tris += stlLocs[0][1]
    #stlのlistを作成
    [nList, sList, lineList] = cm.getStlNodeSurfaceEdgeList(tris)
    #面同士の角度からedgeを抽出
    edgeLineNos = getEdgeLinesFromStl(nList, sList, lineList)

    #分割面のsurNo(maxNo)を取得
    sepSurNo = surfaceList[1][0][0]
    for i in range(1, len(surfaceList)):
        if sepSurNo < surfaceList[i][0][0]:
            sepSurNo = surfaceList[i][0][0]

    #nodeAddを取得
    [[minx, miny, minz], [maxx, maxy, maxz]] = minMaxLoc
    #3次元リスト作成
    nodeAdd = [[[ [] for i3 in range(101)] for i2 in range(101)] for i1 in range(101)]
    #表面nodeを取得
    nodes = []
    for i in range(1, len(surfaceList)):
        nodes += surfaceList[i][0]
    nodes = deleteSameData(nodes)
    #addressを作成
    for node in nodes:
        loc = nodeList[node][0]
        [ax, ay, az] = setGrp.getLocAddress(loc, minMaxLoc)
        nodeAdd[ax][ay][az] += [node]

    #座標を移動
    #node移動距離保存用のリスト
    move = [[] for i in range(len(nodeList)+1)]
    #stlのedgeのloop
    for lineNo in edgeLineNos:
        #blockを取得
        node0 = lineList[lineNo][0][0]
        node1 = lineList[lineNo][0][1]
        xx = [nList[node0][0][0], nList[node1][0][0]]
        yy = [nList[node0][0][1], nList[node1][0][1]]
        zz = [nList[node0][0][2], nList[node1][0][2]]
        minx = min(xx); maxx = max(xx)
        miny = min(yy); maxy = max(yy)
        minz = min(zz); maxz = max(zz)
        [ax, ay, az] = setGrp.getLocAddress([minx, miny, minz], minMaxLoc)
        [bx, by, bz] = setGrp.getLocAddress([maxx, maxy, maxz], minMaxLoc)
        #ax, bxを調整（節点を移動させる為、範囲を±2させて広げる）
        [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])   # -1
        #[ax, ay, az] = setGrp.fitIndexA([ax, ay, az])   # -2
        [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])   # +1
        #[bx, by, bz] = setGrp.fitIndexB([bx, by, bz])   # +2
        #block内addressを取得
        blockNodes = setGrp.getLocAddInBlock(nodeAdd, [ax, ay, az], [bx, by, bz])
        #block内のloop
        for node in blockNodes:
            #edgeからの距離算出
            edge = [nList[node0][0], nList[node1][0]]
            point = nodeList[node][0]
            l = geo.isPointInEdge(edge, point)
            if l >= 0.0:
                #faceSizeを算出し、toleranceを決定
                surNo = nodeList[node][2][0]
                surNode0 = surfaceList[surNo][0][1]
                surNode1 = surfaceList[surNo][0][2]
                faceSize = geo.length(nodeList[surNode0][0], nodeList[surNode1][0])
                tol = faceSize * 2**numlevels * 0.5
                if l < tol:
                    #edge上のnodeかcheck
                    if isNodeOnEdge(node, nodeList, surfaceList, sepSurNo) == True:
                        #edge上の場合、移動距離dlを計算
                        dl = geo.vecPointToEdge(edge, point)
                        move[node] += [dl]
    #座標を移動
    for i in range(len(move)):
        if len(move[i]) > 0:
            moveDl = getMinMove(move[i])        #移動距離の最小を選択する
            nodeList[i][0][0] += moveDl[0]
            nodeList[i][0][1] += moveDl[1]
            nodeList[i][0][2] += moveDl[2]
    return nodeList

#
#  getSeparateNodesOfPairMesh
#    目酒分割面の相手側のnodeを取得
def getSeparateNodesOfPairMesh(nodeList, surfaceList, toMeshFile, minMaxLoc, ownNodes, nodeAdd):

    def under0fit0(A):
        for i in range(len(A)):
            if A[i] < 0:
                A[i] = 0
        return A

    def over100fit100(B):
        for i in range(len(B)):
            if B[i] > 100:
                B[i] = 100
        return B

    #相手側のprocNoを取得
    procNo = int(toMeshFile.split(".")[-2])
    #相手側のmeshContsを取得
    [nList, eList, sList, nGrps] = readVolMesh(toMeshFile)
    #相手側の外表面全てのnodeを取得してチェックする
    #  メッシュ読み込み直後は、surfaceGrpNoが全て「2」になっている為、全nodeをチェック
    sepNodes = []
    for surface in sList:
        if len(surface) > 0:
            nodes = surface[0][1:]
            sepNodes += nodes
    sepNodes = deleteSameData(sepNodes)
    #minMaxLoc範囲内のnodeを取得
    selNodes = []
    for node in sepNodes:
        loc = nList[node][0]
        if geo.isPointInBlock(minMaxLoc, loc) == True:
            selNodes.append(node)
    #相手のnodeNoをselNodes内から取得
    matchNodes = []
    #  相手側のnodeのloop
    for nodeNo in selNodes:
        loc = nList[nodeNo][0]
        #表面要素サイズからlocの範囲を計算
        suNo = nList[nodeNo][2][0]
        nodes = sList[suNo][0][1:]
        loc0 = nList[nodes[0]][0]
        loc1 = nList[nodes[1]][0]
        l = geo.length(loc0, loc1) * 0.1
        minLoc = [loc[0]-l, loc[1]-l, loc[2]-l]
        maxLoc = [loc[0]+l, loc[1]+l, loc[2]+l]
        #ax,bxを取得
        [ax, ay, az] = setGrp.getLocAddress(minLoc, minMaxLoc)
        [bx, by, bz] = setGrp.getLocAddress(maxLoc, minMaxLoc)
        #ax, bxを調整
        [ax, ay, az] = under0fit0([ax, ay, az])
        [bx, by, bz] = over100fit100([bx, by, bz])
        [ax, ay, az] = setGrp.fitIndexA([ax, ay, az])
        [bx, by, bz] = setGrp.fitIndexB([bx, by, bz])
        #範囲内のnodeを取得
        blockNodes = setGrp.getLocAddInBlock(nodeAdd, [ax, ay, az], [bx, by, bz])
        #block内(自身のnode)のloop
        lenNodes = []
        for selNode in blockNodes:
            selLoc = nodeList[selNode][0]
            lnn = geo.length(loc, selLoc)
            if lnn < l*1.0e-4:
                lenNodes.append([lnn, selNode])
        #点間距離が最小の相手を選択
        if len(lenNodes) > 0:
            lenNodes.sort()
            selNode = lenNodes[-1][1]
            matchNodes += [[selNode, [procNo, nodeNo]]]
    return matchNodes

#
#  combineSeparateFace
#    分割面をproc毎に整列させる
def combineSeparateFace(sepConts, myProcNo, maxProcNo):
    myProcNo = int(myProcNo)
    conts = [["", []] for i in range(maxProcNo+1)]
    for sepCont in sepConts:
        myNo = sepCont[0]
        procNo = sepCont[1][0]
        nodeNo = sepCont[1][1]
        conts[procNo][0] = procNo
        conts[procNo][1].append([myNo, nodeNo])
    return conts[myProcNo+1:]

#
#  getSeparateNodes
#    メッシュ分割面のnodeに対応する相手側のnodeを取得
def getSeparateNodes(nodeList, surfaceList, volFile, minMaxLoc, procNo, intSurs):
    #分割面のnodeNoと内部surfaceのnodeNoを取得
    #  分割面のsurNoを取得
    maxNo = 0
    for surface in surfaceList:
        if len(surface) > 0:
            surNo = surface[0][0]
            if maxNo < surNo:
                maxNo = surNo
    sepGrpNo = maxNo
    #  分割面と内部surfaceのnodeNoを取得
    sepNodes = []
    for surface in surfaceList:
        if len(surface) > 0:
            surGrp = surface[0][0]
            if (surGrp == sepGrpNo) or (surGrp in intSurs):
                nodes = surface[0][1:]
                sepNodes += nodes
    sepNodes = deleteSameData(sepNodes)
    #nodeAddressを作成
    nodeAdd = setGrp.setNodeAddress(nodeList, sepNodes, minMaxLoc)
    #検索する相手側メッシュファイル名を取得
    myProcNo = int(volFile.split(".")[-2])
    words = volFile.split(".")
    volHeader = ".".join(words[:-2])
    filePattern = volHeader + ".*.vol"
    volFiles = glob.glob(filePattern)
    nums = []
    for vol in volFiles:
        pNo = vol.split(".")[-2]
        nums.append(int(pNo))
    maxProcNo = max(nums)
    toMeshFiles = []
    for p in range(myProcNo+1, maxProcNo+1):
        toMeshFile = volHeader + "." + str(p) + ".vol"
        toMeshFiles.append(toMeshFile)
    #相手メッシュ分割面のnodeを取得
    sepConts = []
    for toMeshFile in toMeshFiles:
        if procNo == "0":
            print "  " + toMeshFile
        sepConts += getSeparateNodesOfPairMesh(nodeList, surfaceList, toMeshFile, minMaxLoc, sepNodes, nodeAdd)
    #各相手メッシュ毎に整列
    sepConts = combineSeparateFace(sepConts, myProcNo, maxProcNo)    
    return sepConts

#
#  createFistrMesh
#    Fistr用のメッシュを作成
def createFistrMesh(nodeList, surfaceList, elementList, elGroups, surGroups, nodeGroups):
    lines = []
    #node行を作成
    lines += ["!NODE" + ls]
    i = 1
    while i < len(nodeList):
        loc = nodeList[i][0]
        words = [str(i), str(loc[0]), str(loc[1]), str(loc[2])]
        line = ", ".join(words) + ls
        lines += [line]
        i += 1
    #element行を作成
    for elGroup in elGroups:
        #header行を作成
        name = elGroup[0]
        numNodes = elGroup[1][0][1:]
        [elType, nodes] = cm.vol2fistr_el(numNodes)
        line = "!ELEMENT, TYPE=" + elType + ", EGRP=" + name + ls
        lines += [line]
        #element内容を作成
        elements = elGroup[1]
        for element in elements:
            words = []
            elNo = str(element[0])
            elNodes = element[1:]
            [name, elNodes] = cm.vol2fistr_el(elNodes)
            for el in elNodes:
                words.append(str(el))
            line = elNo + ", " + ", ".join(words) + ls
            lines += [line]
    #surfaceGroup行を作成
    for surGroup in surGroups:
        surName = surGroup[0]
        surData = surGroup[1]
        header = "!SGROUP, SGRP=" + surName + ls
        lines += [header]
        newSurData = []
        for surNo in surData:
            surNodes = surfaceList[surNo][0][1:]
            surElm = surfaceList[surNo][1][0]       #surElmは、surfaceList[1][0]を取得
            elNodes = elementList[surElm][0][1:]
            [name, surNodes] = cm.vol2fistr_el(surNodes)
            [name, elNodes] = cm.vol2fistr_el(elNodes)
            faceNo = cm.fistr_elFace(surNodes, elNodes)
            newSurData.append([surElm, faceNo])
        newSurData.sort()
        for data in newSurData:
            line = str(data[0]) + ", " + str(data[1]) + ls
            lines += [line]
    #nodeGroupを作成
    for nodeGroup in nodeGroups:
        name = nodeGroup[0]
        data = nodeGroup[1]
        line = "!NGROUP, NGRP=" + name + ls
        lines += [line]
        n = 10
        words = []
        for node in data:
            words.append(str(node))
            n -= 1
            if n <= 0:
                line = ", ".join(words) + ", " + ls
                lines += [line]
                n = 10
                words = []
        if len(words) != 0:
            line = ", ".join(words) + ls
            lines += [line]
    lines += ["!END" + ls]
    return lines

#
#  createGroupsDataForNeutral
#    neutral形式用のgroupgroup行を作成
def createGroupsDataForNeutral(nodeList, surfaceList, elementList, elGroups, surGroups, nodeGroups, separates, myProcNo, elSeparates):
    lines = [ls + "#" + ls]
    lines += ["# ## separate face of processor is maximum surface No." + ls + "#" + ls]
    #elGroupsを作成
    for elGroup in elGroups:
        name = elGroup[0]
        elNo = elGroup[1][0][0]
        elGrpNo = elementList[elNo][0][0]
        line = "# *elementGroup " + str(elGrpNo) + " " + name + ls
        lines += [line]
    #surGroupsを作成
    for surGroup in surGroups:
        name = surGroup[0]
        surNo = surGroup[1][0]
        surGrpNo = surfaceList[surNo][0][0]
        line = "# *surfaceGroup " + str(surGrpNo) + " " + name + ls
        lines += [line]
    #nodeGroupsを作成
    maxNo = 10
    for nodeGroup in nodeGroups:
        name = nodeGroup[0]
        nodeData = nodeGroup[1]
        line = "# *nodeGroup " + name + ls
        lines += [line]
        i = 1
        line = "# "
        for nodeNo in nodeData:
            if i < maxNo:
                line += " " + str(nodeNo)
            else:
                line += " " + str(nodeNo)
                lines += [line + ls]
                i = 0
                line = "# "
            i += 1
        if line != "# ":
            lines += [line + ls]
    #elementGrp間の境界面のsurGrpNoを作成
    line = "# *internalSurfaceNo    <surfaceNo between element groups>" + ls
    lines += [line]
    maxNo = 10
    i = 1
    line = "# "
    for surNo in elSeparates:
        if i < maxNo:
            line += " " + str(surNo)
        else:
            line += " " + str(surNo)
            lines += [line + ls]
            i = 0
            line = "# "
        i += 1
    if line != "# ":
        lines += [line + ls]
    #separate面のデータを作成
    maxNo = 5
    for i in range(len(separates)):
        procNo = i + myProcNo + 1
        nodeData = separates[i][1]
        line = "# *toProc_" + str(procNo) + "   <myProcNode>:<proc" + str(procNo) + "Node>" + ls
        lines += [line]
        i = 1
        line = "# "
        for data in nodeData:
            if i < maxNo:
                line += " " + str(data[0]) + ":" + str(data[1])
            else:
                line += " " + str(data[0]) + ":" + str(data[1])
                lines += [line + ls]
                i = 0
                line = "# "
            i += 1
        if line != "# ":
            lines += [line + ls]
    return lines

#
#  createNeutralMesh
#    neutral形式のメッシュファイルを作成
def createNeutralMesh(nodeList, surfaceList, elementList, elGroups, surGroups, nodeGroups, separates, myProcNo, elSeparates):
    lines = cm.createNeutralMeshLines(nodeList, surfaceList, elementList)
    lines += createGroupsDataForNeutral(nodeList, surfaceList, elementList, elGroups, surGroups, nodeGroups, separates, myProcNo, elSeparates)
    return lines

#
#  printHelp
#
def printHelp():
    cont = """
----------- getGroupsFromVolStl.py の使い方 ------------------
メッシュ分割されたvol形式のメッシュにnodeGroupやsurfaceGroupを設定する。
premesh（分割前メッシュ）がelement, surface, nodeの各グループ分けされて
いる場合は、そのgroupを作成する。
nodeGroup、surfaceGroupについては、その形状のstlファイルを準備して、
file名を指定することで、作成できる。

＜使い方＞
getGroupsFromVolStl.py <premesh> <partMesh> <numlevels> [option]
    premesh         分割前のメッシュfile（vol形式）
    partmesh        分割後のメッシュfile（vol形式）
    numlevels       pmsh_netgenのnumlevels（細分化level）

[option]
    --ndGrps        nodeGroupを作成するためのstlファイルを指定（省略可）
                    stlファイルを複数指定可能
    --sfGrps        surfaceGroupを作成するためのstlファイルを指定（省略可）
                    stlファイルを複数指定可能
    --fit           曲面の表面形状をfitさせる為のstlファイルを指定（省略可）
                    stlファイルを複数指定可能。
    --help          helpを表示

＜使用例＞
getGroupsFromVolStl.py premesh.vol mesh.0.vol 3 --fit sideFace.stl
getGroupsFromVolStl.py premesh.vol mesh.0.vol 3 --ndGrps topFace.stl bottomFace.stl --sfGrps side.stl

"""
    print cont

#
#  printP0
#    procNo=0の時出力
def printP0(procNo, msg):
    if procNo == "0" or procNo == 0:
        print msg

#
#  getNodeGroupFromVolStl
#  -------------------
#    volメッシュからstl形状のnodeGroupを取得する
#def getNodeGroupFromVolStl(volFile, stlFiles):
def getNodeGroupFromVolStl(premeshFile, volFile, ndStlFiles, sfStlFiles, fitFiles):
    #変数の初期化
    procNo = volFile.split(".")[-2]
    elGroups = []
    surGroups = []
    nodeGroups = []
    minMaxLoc = []
    #preMeshとvolからメッシュ内容を取得する
    printP0(procNo, ls + "Remake each part meshes")
    printP0(procNo, "reading premesh and partmesh files...")

    printP0(procNo, "  reading premesh file...")
    [preMeshList, preTri, preTet, internalSurs, preGrps] = setGrp.readPreMeshConts(premeshFile)
    printP0(procNo, "  reading processor mesh file...")
    [nodeList, elementList, surfaceList, nGrps] = readVolMesh(volFile)
    printP0(procNo, "  getting surface groups and separated faces of premesh...")
    minMaxLoc = setGrp.getMinMaxLocation(nodeList)
    surfaceList = setGrp.setSurfaceGroups(nodeList, surfaceList, preTri, minMaxLoc)
    printP0(procNo, "  getting element groups of premesh...")
    elementList = setGrp.setElementGroups(nodeList, elementList, preTet, minMaxLoc)

    #メッシュ内容から各groupをから取得
    printP0(procNo, "setting each group of premesh...")
    elGroups = createElementGroupsFromList(elementList, preGrps)
    if len(preGrps) == 0:
        [surGroups, nodeGroups] = createSurfaceNodeGroupsFromList(surfaceList)
    else:
        meshList = [nodeList, elementList, surfaceList]
        [surGroups, nodeGroups] = createSurfaceNodeGroupsFromPremesh(preMeshList, preGrps, meshList, minMaxLoc)
    #nodeGroup, surfaceGroup取得する場合、surfaceのaddressを取得する
    if len(ndStlFiles) != 0 or len(sfStlFiles) != 0 or len(fitFiles) != 0:
        surAdd = setGrp.setSurfaceAddress(nodeList, surfaceList, minMaxLoc)
    #nodeGroupを取得する
    if len(ndStlFiles) != 0:
        printP0(procNo, "creating node groups from stl files...")
        #nodeGroupsを取得し、入れ替え
        stlLocs = getStlTriLoc(ndStlFiles)
        nodeGroups = getNodeGroupsFromStl(nodeList, surfaceList, stlLocs, minMaxLoc, surAdd)
    #surfaceGroupを取得する
    if len(sfStlFiles) != 0:
        #surfaceGroupを取得し、入れ替え
        printP0(procNo, "creating surface groups from files...")
        stlLocs = getStlTriLoc(sfStlFiles)
        surGroups = getSurGroupsFromStl(nodeList, surfaceList, stlLocs, minMaxLoc, surAdd)
        #surfaceListを再作成
        surfaceList = remakeSurfaceList(surfaceList, surGroups)
    #分割面の相手側nodeを取得
    printP0(procNo, "getting nodes of separated face...")
    separates = getSeparateNodes(nodeList, surfaceList, volFile, minMaxLoc, procNo, internalSurs)
    #surfaceをfitさせる
    if len(fitFiles) != 0:
        #表面nodeを移動させる
        printP0(procNo, "fitting nodes to stl files...")
        stlLocs = getStlTriLoc(fitFiles)
        printP0(procNo, "  fitting face...")
        nodeList = moveNodeLocToFitFile(nodeList, surfaceList, stlLocs, minMaxLoc, surAdd)
        printP0(procNo, "  fitting edge...")
        nodeList = moveNodeToEdgeOfFitFile(nodeList, surfaceList, stlLocs, minMaxLoc)
    #neutral形式で保存
    printP0(procNo, "saving mesh at Neutral format...")
    lines = createNeutralMesh(nodeList, surfaceList, elementList, elGroups, surGroups, nodeGroups, separates, int(procNo), internalSurs)
    fileName = ".".join(volFile.split(".")[:-1]) + ".mesh"
    writeFileLines(fileName, lines)
    printP0(procNo, "  ... done.")
    #Fistr形式メッシュを作成、保存
    #if procNo == "0":
    lines = createFistrMesh(nodeList, surfaceList, elementList, elGroups, surGroups, nodeGroups)
    writeFileLines("FistrModel."+procNo+".msh", lines)

#
#  getArgument
#    引数を取得
def getArgument():
    premesh = ""
    vol = ""
    ndGrps = []
    sfGrps = []
    fit = []
    if len(sys.argv) < 4:
        printHelp()
        return []
    premesh = sys.argv[1]
    vol = sys.argv[2]
    try:
        numlevels = int(sys.argv[3])
    except:
        printHelp()
        return []
    flag = ""
    for i in range(4, len(sys.argv)):
        if sys.argv[i][0] == "-":
            if sys.argv[i] == "--ndGrps" or sys.argv[i] == "-ndGrps":
                flag = "ndGrps"
            elif sys.argv[i] == "--sfGrps" or sys.argv[i] == "-sfGrps":
                flag = "sfGrps"
            elif sys.argv[i] == "--fit" or sys.argv[i] == "-fit":
                flag = "fit"
            elif sys.argv[i] == "--help" or sys.argv[i] == "-help":
                flag = "help"
            else:
                flag = "help"
        else:
            if flag == "ndGrps":
                ndGrps.append(sys.argv[i])
            elif flag == "sfGrps":
                sfGrps.append(sys.argv[i])
            elif flag == "fit":
                fit.append(sys.argv[i])
    if flag == "help":
        printHelp()
        return []
    return [premesh, vol, numlevels, ndGrps, sfGrps, fit]


if __name__ == "__main__":
    import gettext
    gettext.install("app")
    args = getArgument()
    if len(args) == 0:
        exit()

    [premeshFile, volFile, numlevels, ndStlFiles, sfStlFiles, fitFiles] = args
    getNodeGroupFromVolStl(premeshFile, volFile, ndStlFiles, sfStlFiles, fitFiles)

    

