#!/usr/bin/env python
# -*- coding: utf-8 -*-    
#
#   paraMesh.py
#
#       vol形式のpremeshを分割して並列処理でメッシュを作成する
#
#   18/03/01    新規作成
#

import sys
import os
import glob

ls = "\n"


#
#  printHelp
#
def printHelp():
    cont = """
------------- paraMesh.py の使い方 -------------------------------------------
並列処理でメッシュを作成する。大規模メッシュ作成に使用できる。
メッシュの作成方法は、予め、粗いメッシュをvol形式の粗いメッシュを作成しておき、
paraMesh.pyが、その粗いメッシュを各process毎に分割して、各processが、細かく
メッシュを切り直す方法。
メッシュを細かく切り直す時、表面の節点が追加されるが、追加された節点の位置は、
表面形状のstlファイルを準備することで、その形状の位置に移動させる。
nodeGroup、surfaceGroupを作成する場合は、その形状のstlファイルを準備する。
elementGroupは、粗いメッシュの状態で、group化しておく。

＜使い方＞
paraMesh.py -np <nProc> -nl <numLevel> -pre <premesh> [option <parameter>]
    -np <nProcs>    並列数を設定
    -nl <numLevel>  メッシュ再分割化のレベル
    -pre <premesh>  vol形式の粗いメッシュfileを指定
    -maxh <maxSize> メッシュを細かく切り直す時の内部の最大メッシュサイズを指定
                    default値は、「1000」
    -nd <stl>       stlファイルの形状のnodeGroupを作成する。（複数指定可）
    -sf <stl>       stlファイルの形状のsurfaceGroupを作成する（複数指定可）
    -f <stl>        節点の位置をstlファイルの形状の位置に移動（複数指定可）
    -h              ヘルプ
    --np            -npと同じ
    --numlevel      -nlと同じ
    --premesh       -preと同じ
    --maxh          -maxhと同じ
    --ndGrps        -ndと同じ
    --sfGrps        -sfと同じ
    --fit           -fと同じ
    --help          -hと同じ

＜使用例＞
paraMesh.py -np 4 -nl 2 -pre premesh.vol -f sideFace.stl
paraMesh.py -np 4 -nl 2 -pre premesh.vol -nd topFace.stl botomFace.stl -sf sideFace.stl -f sideFace.stl
-------------------------------------------------------------------------------
"""
    print cont

#
#  getArgument
#    引数を取得
def getArgument():
    premesh = ""
    ndGrps = []
    sfGrps = []
    fit = []
    np = -1
    nl = -1
    maxh = "1000"
    flag = ""
    if len(sys.argv) < 4:
        printHelp()
        return []

    for i in range(1, len(sys.argv)):
        if sys.argv[i][0] == "-":
            if sys.argv[i] == "--premesh" or sys.argv[i] == "-pre":
                flag = "premesh"
            elif sys.argv[i] == "--maxh" or sys.argv[i] == "-maxh":
                flag = "maxh"
            elif sys.argv[i] == "--ndGrps" or sys.argv[i] == "-nd":
                flag = "ndGrps"
            elif sys.argv[i] == "--sfGrps" or sys.argv[i] == "-sf":
                flag = "sfGrps"
            elif sys.argv[i] == "--fit" or sys.argv[i] == "-f":
                flag = "fit"
            elif sys.argv[i] == "--help" or sys.argv[i] == "-h":
                flag = "help"
            elif sys.argv[i] == "--nproc" or sys.argv[i] == "-np":
                flag = "np"
            elif sys.argv[i] == "--numlevel" or sys.argv[i] == "-nl":
                flag = "nl"
        else:
            if flag == "premesh":
                premesh = sys.argv[i]
            elif flag == "ndGrps":
                ndGrps.append(sys.argv[i])
            elif flag == "sfGrps":
                sfGrps.append(sys.argv[i])
            elif flag == "fit":
                fit.append(sys.argv[i])
            elif flag == "np":
                try:
                    np = sys.argv[i]
                    val = int(np)
                except:
                    flag = "error"
                    break
            elif flag == "nl":
                try:
                    nl = sys.argv[i]
                    val = int(nl)
                except:
                    flag = "error"
                    break
            elif flag == "maxh":
                try:
                    maxh = sys.argv[i]
                    val = float(maxh)
                except:
                    flag = "error"
                    break
            elif flag == "help":
                break
    if flag == "help" or flag == "error":
        printHelp()
        return []
    if premesh == "" or np == -1 or nl == -1:
        print Help()
        return []
    return [premesh, ndGrps, sfGrps, fit, np, nl, maxh]

#
#  errorCheck
#    file有無チェック
def errorCheck(premesh, ndGrps, sfGrps, fit):

    def errorMsg(fileName):
        print ls + "ERROR: does not exist '" + fileName + " !!" + ls
        printHelp()

    def errorCheckFiles(grps):
        stat = "OK"
        for fileName in grps:
            if len(glob.glob(fileName)) == 0:
                errorMsg(fileName)
                stat = "NG"
                break
        return stat

    stat = "OK"
    if len(glob.glob(premesh)) == 0:
        errorMsg(premesh)
        return "NG"
    if errorCheckFiles(ndGrps) == "NG":
        return "NG"
    if errorCheckFiles(sfGrps) == "NG":
        return "NG"
    if errorCheckFiles(fit) == "NG":
        return "NG"
    return "OK"

#
#  deleteTempFiles
#    pmesh*.vol, pmesh*.mesh を削除する
def deleteTempFiles():
    delFiles = glob.glob("pmesh.*.vol")
    delFiles += glob.glob("pmesh.*.mesh")
    for delFile in delFiles:
        os.remove(delFile)

#
#  createCommLines
#    スクリプト作成
def createCommLines(premesh, ndGrps, sfGrps, fit, np, nl, maxh):
    lines = []
    lines += ["#!/bin/bash" + ls]
    lines += [". bashrc-netgen" + ls]
    line = "mpirun -np " + np + " pmsh_netgen --premesh " + premesh + " --stl pmesh --numprocs " + np + " --fineness 1.0 --maxh " + maxh + " --numlevels " + nl + " --outvol" + ls
    lines += [line]
    nodeGrps = ""
    if len(ndGrps) > 0:
        nodeGrps = "--ndGrps "
        for i in range(len(ndGrps)):
            nodeGrps += ndGrps[i] + " "
    surGrps = ""
    if len(sfGrps) > 0:
        surGrps = "--sfGrps "
        for i in range(len(sfGrps)):
            surGrps += sfGrps[i] + " "
    fits = ""
    if len(fit) > 0:
        fits = "--fit "
        for i in range(len(fit)):
            fits += fit[i] + " "
    for i in range(int(np)):
        line = "getGroupsFromVolStl.py " + premesh + " pmesh." + str(i) + ".vol " + nl + " " + nodeGrps + surGrps + fits + "&" + ls
        lines += [line]
    lines += ["wait" + ls]
    lines += ["echo '... all done'" + ls]
    #書き込み
    fileName = "run"
    f = open(fileName, "w")
    for line in lines:
        f.write(line)
    f.close()
    os.system("chmod a+x run")



if __name__ == "__main__":
    import gettext
    gettext.install("app")

    args = getArgument()
    if len(args) == 0:
        exit()

    [premesh, ndGrps, sfGrps, fit, np, nl, maxh] = args
    stat = errorCheck(premesh, ndGrps, sfGrps, fit)
    if stat == "OK":
        deleteTempFiles()
        createCommLines(premesh, ndGrps, sfGrps, fit, np, nl, maxh)
        os.system("./run")


